@@ -1,6 +1,10 @@
 /* 
- * Copyright (C) 1995-2013 Index Data
- * 
+ * Copyright (C) 2013 Index Data
+ *
+ * Unit tests for the filter_cproxy
+ *
+ * For the content-connector tests, tries to make SRU requests
+ * to port 29748, on the assumption that nothing is likely to listen there
  */
 
 #if HAVE_CONFIG_H
@@ -41,6 +45,10 @@ mp::odr odr;
 // Comment out to clean up things properly
 #define KEEP_DUMP_DIRS 1  
 
+// Define a port number to make the content-connector SRU requests to.
+// These are supposed to fail, so nothing should be listening there.
+#define CFENGINEPORT "9768"
+
 //////////////////////////
 // Helpers to check things
 
@@ -131,7 +139,7 @@ static bool check_response_header( mp::Package *pack,
         return false;
     }
     return true;
-} // check_request_header
+} // check_response_header
 
 // Check a that request header exists, and contains the fragment
 static bool check_request_header( mp::Package *pack,
@@ -256,8 +264,15 @@ mp::filter::CProxy *make_filter( const char *xmlconf)
     assert(doc);
     xmlNode *root_element = xmlDocGetRootElement(doc);
     assert(root_element);
-    cpx->configure(root_element, true, "");
-    xmlFreeDoc(doc);
+    try {
+        cpx->configure(root_element, true, "");
+        xmlFreeDoc(doc);
+    }
+    catch (std::exception & e) {
+        delete cpx;
+        xmlFreeDoc(doc);
+        throw(e);
+    }
     return cpx;
 } // make_filter
 
@@ -312,6 +327,7 @@ std::string url_enc( std::string comp )
     return std::string(ubuf);
 }
 
+
 /////////////////////////
 // Helpers to make files
 // We keep a (global) list of files we have created, so we can delete them
@@ -352,7 +368,9 @@ void make_param_file( std::string sesdir,
                       std::string connector,
                       std::string auth,
                       std::string proxy,
-                      std::string realm )
+                      std::string realm,
+                      std::string cfrepoauth = ""
+                    )
 {
     std::string name = sesdir + "/cf." + sesno + ".p" ;  // "./cf.17.p"
     std::ofstream f(name.c_str());
@@ -363,6 +381,9 @@ void make_param_file( std::string sesdir,
     f << "auth: " << auth << "\n";
     f << "proxy: " << proxy << "\n";
     f << "realm: " << realm << "\n";
+    if ( ! cfrepoauth.empty() ) {
+        f << "cfrepoauth: " << cfrepoauth << "\n";
+    }
     f.close();
     tempfilelist.push_back(name);    
 } // make_param_file 
@@ -428,11 +449,27 @@ void test1(void)
     const char * conf =
         "<?xml version='1.0'?>\n"
         "<filter type='cproxy'>\n"
-        "  <debug>1</debug>\n"    // gets enabled via the URL when needed
+        "  <debug>0</debug>\n"    // gets enabled via the URL when needed
         "  <cfconfig>./cf-config.unittest.cfg</cfconfig>\n"
         "  <sessionmaxage>1</sessionmaxage>\n"
-        "  <disableposttrick>^disableposttrick.com/.*</disableposttrick>\n"
-        "  <disableposttrick>^.*/disableposttrick/</disableposttrick>\n"
+        "  <omitcookie>\n"
+        "    <domain>omit1.com</domain>\n"
+        "  </omitcookie>\n" // minimal line
+        "  <omitcookie>\n"
+        "    <method>GET</method>\n"
+        "    <domain>omit2.com</domain>\n"
+        "    <names>+yes.* .*</names>\n"
+        "  </omitcookie>\n"
+        "  <skiplink>\n"
+        "    <page>skippage.com</page>\n"
+        "  </skiplink>\n"
+        "  <skiplink>\n"
+        "    <link>skiplink.com</link>\n"
+        "  </skiplink>\n"
+        "  <skiplink>\n"
+        "    <page>bothpage.com</page>\n"
+        "    <link>bothlink.com</link>\n"
+        "  </skiplink>\n"
         "</filter>\n";
     make_cf_config("./cf-config.unittest.cfg", // config file to write
                    "localhost:9036/XXX/node102",  // url prefix
@@ -492,8 +529,6 @@ void test1(void)
              "Creating ./cf.99.dump/badsession.com_error.txt failed with 2" ) );
     YAZ_CHECK( check_response_content(pack, "Could not open session file ./cf.99" ) );
     YAZ_CHECK( check_response_content(pack, "Error 400 No session" ) );
-    YAZ_CHECK( check_response_content(pack,
-             "No cfengine specified, not even looking at content connectors" ) );
     delete(pack);
 
     // Existing session file, but pretty much emtpy. No cookies in request
@@ -508,6 +543,13 @@ void test1(void)
     YAZ_CHECK( check_response_content(pack, "Original request cookie line ''" ) );
     YAZ_CHECK( check_response_content(pack, "Final cookie line: ''" ) );
     YAZ_CHECK( check_no_request_header(pack, "Cookie") );
+    // Check that we pass the skiplink headers from the config
+    YAZ_CHECK( check_any_request_header(pack,"X-Metaproxy-SkipLink",
+                                        "skippage.com .*" ));
+    YAZ_CHECK( check_any_request_header(pack,"X-Metaproxy-SkipLink",
+                                        ".* skiplink.com" ));
+    YAZ_CHECK( check_any_request_header(pack,"X-Metaproxy-SkipLink",
+                                        "bothpage.com bothlink.com" ));
     delete(pack);
 
     // Existing session file, but pretty much emtpy.
@@ -693,41 +735,20 @@ void test1(void)
     YAZ_CHECK( !check_response_code(pack, 302) );   // not a redirect!
     delete(pack);
 
-    // Similar bad POST request, but to an address where we do not want to
-    // apply the test. CPXY-89
-    pack = make_package("http://localhost:9036/cproxydebug-nomove-verbose/"
-               "page.txt?param=value",1,"POST",content);
-    add_request_header(pack, "Referer",
-         "http://localhost:9036/XXX/node102/100/"
-         "disableposttrick.com/ref.path/ref.to/ref.page.html");
-    cpx->process(*pack);
-    YAZ_CHECK( check_response_code(pack, 400) );   // Bad req, no session
-    delete(pack);
-
-    // Same again, but with a different regexp. Also checks that we can loop
-    // through the pattern list.
-    pack = make_package("http://localhost:9036/cproxydebug-nomove-verbose/"
-               "foo.com/disableposttrick/page.txt?param=value",1,"POST",content);
-    add_request_header(pack, "Referer",
-         "http://localhost:9036/XXX/node102/100/"
-         "ref.host/ref.path/ref.to/ref.page.html");
-    cpx->process(*pack);
-    YAZ_CHECK( check_response_code(pack, 400) );   // Bad req, no session
-    delete(pack);
-
-    
     remove_dump_dir("./cf.100.dump");
 
-    // More complex session
+    // More complex session. Now (as of 3.1.0) with new-form cookie lines.
     make_session_file( "." , "101",
               "Referer http://www.indexdata.com/software/ \n"
               "Username someuser\n"
               "Password secretpassword\n"
               "Proxyip 1.2.3.4\n"
-              "Cookie www.indexdata.com; one=ONE\n"
-              "Cookie .indexdata.com; two=TWO\n"
-              "Cookie other.indexdata.com; three=THREE\n"
-              "Cookie WwW.InDeXdAtA.cOm; mixedcase=true\n" );
+              "Baseurl http://www.indexdata.com/somepage?with=two&params=x\n"
+              "Set-Cookie one=ONE; domain = www.indexdata.com \n"
+              "Set-Cookie two=TWO; Domain=.indexdata.com; \n"
+              "Set-Cookie three=THREE; domain=other.indexdata.com\n"
+              "Set-Cookie  mixedcase=true; domain=WwW.InDeXdAtA.cOm\n"
+              "SkipLink skippage2.com skiplink2.com\n" );
     // The cookie called mixedcase is to check that the domain matching is
     // indeed case-insensitive (CP-3482)
     pack = process_url(cpx, "http://localhost:9036/XXX/cproxydebug-7/node102/"
@@ -741,6 +762,10 @@ void test1(void)
     YAZ_CHECK( check_request_header(pack, "Cookie","two=TWO;one=ONE") );
     YAZ_CHECK( check_request_header(pack, "Referer","http://www.indexdata.com/software/") );
     YAZ_CHECK( check_request_header(pack, "X-Metaproxy-Proxy", "1.2.3.4") );
+    YAZ_CHECK( check_request_header(pack, "Authorization",
+                                    "Basic c29tZXVzZXI6c2VjcmV0cGFzc3dvcmQ=") );
+    YAZ_CHECK( check_any_request_header(pack,"X-Metaproxy-SkipLink",
+                                        "skippage2.com skiplink2.com" ));
     
     delete(pack);
 
@@ -755,14 +780,24 @@ void test1(void)
     YAZ_CHECK( check_request_header(pack, "Cookie",
                             "one=first;four=fourth;mixedcase=true;two=TWO") );
     delete(pack);
+    // Second request, to indexdata.dk. Will not match baseurl, so no auth
+    pack = make_package("http://localhost:9036/XXX/cproxydebug-verbose-nomove-dump/node102/"
+                        "101/www.indexdata.dk/staff/",0);
+    cpx->process(*pack);
+    YAZ_CHECK( check_response_code(pack, 200) );
+    YAZ_CHECK( check_no_request_header(pack, "Authorization") );
+    delete(pack);
 
-    // New-form cookie lines
+    // Session with new-form cookie lines, username without password
+    // also a Basicauthwith, that forces auth to .dk and not to .com
     make_session_file( "." , "102",
-              "Cookie www.indexdata.com; one=ONE\n"
               "Set-Cookie newcookie=newvalue;\n"
               "Set-Cookie newcookie2=v2;Domain=www.indexdata.com;Path=/;\n"
               "Set-Cookie newcookie3=v3;Domain=www.indexdata.com;Path=/staff;\n"
               "Set-Cookie newcookie4=v4;Domain=www.indexdata.com;Path=/about;\n"
+              "Username someuser\n"
+              "Baseurl http://www.indexdata.com/somepage?with=two&params=x\n"
+              "Basicauthwith www.indexdata.dk\n"
                      );
     pack = make_package("http://localhost:9036/XXX/cproxydebug-verbose-nomove-dump/node102/"
                         "102/www.indexdata.com/staff/",0);
@@ -772,10 +807,19 @@ void test1(void)
     YAZ_CHECK( check_response_code(pack, 200) );
     YAZ_CHECK( check_response_content(pack, "Got session '102'" ) );
     YAZ_CHECK( check_response_content(pack, "New cookie line name='newcookie' value='newvalue'" ) );
+    YAZ_CHECK( check_no_request_header(pack, "Authorization") );
 
     YAZ_CHECK( check_request_header(pack, "Cookie", // no newcookie4, wrong path
             "one=first;four=fourth;newcookie3=v3;newcookie2=v2;newcookie=newvalue") );
+    delete(pack);   
+    // Second request, to indexdata.dk. Will match basicauthwith, so expect auth
+    pack = make_package("http://localhost:9036/XXX/cproxydebug-verbose-nomove-dump/node102/"
+                        "102/www.indexdata.dk/staff/",0);
+    cpx->process(*pack);
+    YAZ_CHECK( check_response_code(pack, 200) );
+    YAZ_CHECK( check_request_header(pack, "Authorization", "Basic c29tZXVzZXI6") );
     delete(pack);
+
     remove_dump_dir("./cf.102.dump");
 
 
@@ -785,7 +829,7 @@ void test1(void)
     make_cf_config("./cf-config.unittest.cfg", // config file to write
                    "localhost:9036/XXX/node102",  // url prefix
                    ".", // session directory
-                   "localhost:9001" );  // cf-zserver
+                   "localhost:" CFENGINEPORT );  // cf-zserver
     delete(cpx);
     cpx = make_filter( conf );   // read the new config with a cf-zserver
     make_param_file(".", "103",
@@ -808,22 +852,25 @@ void test1(void)
                     "fake_content_connector",
                     "user/passwd",
                     "1.2.3.4:80",
-                    "testrealm" );
+                    "testrealm",
+                    "cfusername/cfpassword"
+                   );
 
     pack = process_url(cpx, "http://localhost:9036/XXX/cproxydebug-7/node102/"
                         "103/www.indexdata.com/staff/");
     YAZ_CHECK( check_response_code(pack, 400 ) );
     YAZ_CHECK( check_response_content(pack, "Making SRU request "
-        "http://localhost:9001/fake_content_connector"
-            ",cproxysession=103"
-            "&realm=testrealm"
-            "&user=user"
-            "&password=passwd"
-            "&proxy=1.2.3.4:80"
-          "?version=1.2"
-          "&operation=searchRetrieve"
-          "&x-pquery=dummy"
-          "&maximumRecords=0" ) );
+        "http://cfusername:cfpassword@localhost:" CFENGINEPORT
+            "/fake_content_connector"
+                ",cproxysession=103"
+                "&realm=testrealm"
+                "&user=user"
+                "&password=passwd"
+                "&proxy=1.2.3.4%3A80"    // note url-encoded :80
+            "?version=1.2"
+            "&operation=searchRetrieve"
+            "&x-pquery=dummy"
+            "&maximumRecords=0" ) );
     delete(pack);
 
     // Same again, with only username, no password, no realm
@@ -831,7 +878,7 @@ void test1(void)
     // it can be used. This needs to be a manual test:
     //   - Edit your /etc/cf-proxy/cproxy.cfg
     //     - Point the sessiondir to the directory you run this test in
-    //   - Start a cf-zserver: cd .../cf/engine/src; ./cf-zserver @:9001
+    //   - Start a cf-zserver: cd .../cf/engine/src; ./cf-zserver @:CFENGINEPORT
     //   - make unittest. This reports FAILURE
     //   - Look at the output, you should see a searchRetrieveResponse
     //     with zero hits.
@@ -851,16 +898,38 @@ void test1(void)
                         "104/www.indexdata.com/staff/");
     YAZ_CHECK( check_response_code(pack, 400 ) );
     YAZ_CHECK( check_response_content(pack, "Making SRU request "
-        "http://localhost:9001/cf-zserver-regression/proxy-url-in-init"
+        "http://localhost:" CFENGINEPORT "/cf-zserver-regression/proxy-url-in-init"
             ",cproxysession=104"
             "&user=user"
-            "&proxy=1.2.3.4:80"
+            "&proxy=1.2.3.4%3A80"
           "?version=1.2"
           "&operation=searchRetrieve"
           "&x-pquery=dummy"
           "&maximumRecords=0" ) );
     delete(pack);
 
+    // Creating a session with the help of a param file with a null content connector
+    // This will not fail, since the null connector is not passed to the CF,
+    // instead the auth and proxy params are taken directly
+    make_param_file(".", "105",
+                    "null",
+                    "user/passwd",
+                    "1.2.3.4:80",
+                    "testrealm",
+                    "cfusername/cfpassword"
+                   );
+
+    pack = process_url(cpx, "http://localhost:9036/XXX/cproxydebug-7/node102/"
+                        "105/www.indexdata.com/staff/");
+    YAZ_CHECK( check_response_code(pack, 200 ) );
+    YAZ_CHECK( check_response_content(pack, "Faking a null session") );
+    YAZ_CHECK( check_request_header(pack, "X-Metaproxy-Proxy", "1.2.3.4:80") );
+    YAZ_CHECK( check_request_header(pack, "Authorization", "Basic dXNlcjpwYXNzd2Q=") );
+
+    delete(pack);
+
+
+
     // A cf-config file with a bad proxy prefix (all numerical component)
     // Those will get confused with the session number, so they are disallowed
     // (actually, filter_cproxy will handle the just fine, but later a
@@ -870,7 +939,7 @@ void test1(void)
     make_cf_config("./cf-config.unittest.cfg", // config file to write
                    "localhost:9036/XXX/2013",  // Illegal url prefix
                    ".", // session directory
-                   "localhost:9001" );  // cf-zserver
+                   "localhost:" CFENGINEPORT );  // cf-zserver
     try {
         mp::filter::CProxy *bad_cpx = make_filter( conf );
         YAZ_CHECK (!"Did not throw an exception for a bad proxyprefix!");
@@ -879,6 +948,59 @@ void test1(void)
         YAZ_CHECK ("got an exception as expected" );
     }
 
+    // Omitting some cookies
+    // domains omit1 and omit2 come from the config, 3 and 4 from the session here
+    // (note the \s between the method and the domain, that's a regexp space)
+    // Also, username and password, no baseurl nor basicauthwith lines. This is
+    // to test we still keep the old behavior, and add auth to everything
+    make_session_file( "." , "103",
+              "OmitCookie omit3.com \n"
+              "OmitCookie GET\\somit4.com +yes.* .*\n"
+              "Username someuser\n"
+              "Password secretpassword\n"
+                     );
+    pack = make_package("http://localhost:9036/XXX/cproxydebug-verbose-nomove-dump/node102/"
+                        "103/omit1.com/staff/",0);
+    add_request_header(pack, "Cookie","one=first;");
+    cpx->process(*pack);
+
+    YAZ_CHECK( check_response_code(pack, 200) );
+    YAZ_CHECK( check_response_content(pack, "Got session '103'" ) );
+
+    // Cookie should be omitted
+    YAZ_CHECK( !check_request_header(pack, "Cookie",  "one=first") );
+    // Should have auth header, no matter what domain
+    YAZ_CHECK( check_request_header(pack, "Authorization", "Basic c29tZXVzZXI6") );
+    
+    delete(pack);
+
+    pack = make_package("http://localhost:9036/XXX/cproxydebug-verbose-nomove-dump/node102/"
+                        "103/omit2.com/staff/",0);
+    add_request_header(pack, "Cookie","yescookie=good;nocookie=bad;");
+    cpx->process(*pack);
+    YAZ_CHECK( check_request_header(pack, "Cookie",  "yescookie=good") );
+    YAZ_CHECK( !check_request_header(pack, "Cookie",  "nocookie") );
+    YAZ_CHECK( check_request_header(pack, "Authorization", "Basic c29tZXVzZXI6") );
+    delete(pack);
+
+    pack = make_package("http://localhost:9036/XXX/cproxydebug-verbose-nomove-dump/node102/"
+                        "103/omit3.com/staff/",0);
+    add_request_header(pack, "Cookie","yescookie=good;nocookie=bad;");
+    cpx->process(*pack);
+    YAZ_CHECK( !check_request_header(pack, "Cookie",  "=") );
+    delete(pack);
+
+    pack = make_package("http://localhost:9036/XXX/cproxydebug-verbose-nomove-dump/node102/"
+                        "103/omit4.com/staff/",0);
+    add_request_header(pack, "Cookie","yescookie=good;nocookie=bad;");
+    cpx->process(*pack);
+    YAZ_CHECK( !check_request_header(pack, "Cookie",  "one=first") );
+    YAZ_CHECK( check_request_header(pack, "Cookie",  "yescookie=good") );
+    YAZ_CHECK( !check_request_header(pack, "Cookie",  "nocookie") );
+    delete(pack);
+
+    remove_dump_dir("./cf.103.dump");
+
     delete(cpx);
 } // test1
 
@@ -914,6 +1036,25 @@ void test2 (void)
                     "<debug>0</debug>\n"    // will be enabled as needed
                     "<cfconfig>./cf-config.unittest2.cfg</cfconfig>\n"
                     "<sessionmaxage>1</sessionmaxage>\n"
+                    "<cookiedomainexpansion>\n"
+                        "<domain>.wc1.com</domain>\n"
+                        "<expansion>www auth</expansion>\n"
+                    "</cookiedomainexpansion>\n"
+                    "<cookiedomainexpansion>\n"
+                        "<domain>.wc3.com</domain>\n"
+                        "<expansion>www auth</expansion>\n"
+                    "</cookiedomainexpansion>\n"
+                    "<cookiedomainexpansion>\n"
+                        "<domain>.wc4.com</domain>\n"
+                        "<expansion>www auth</expansion>\n"
+                    "</cookiedomainexpansion>\n"
+                    "<skiplink>\n"
+                        "<page>skippage.com</page>\n"
+                    "</skiplink>\n"
+                    "<skiplink>\n"
+                        "<link>skiplink.com</link>\n"
+                    "</skiplink>\n"
+                    
                 "</filter>\n"
 
 #ifdef LOG_ALL
@@ -995,6 +1136,7 @@ void test2 (void)
     xmlDocPtr doc = xmlParseMemory(conf.c_str(), conf.size());
     mp::RouterXML router(doc,false, "../etc/" );
     YAZ_CHECK("configured the router chain all right");
+    xmlFreeDoc(doc);
     
     std::string page1 = 
       "<html><body>\n"
@@ -1071,6 +1213,7 @@ void test2 (void)
       "<script>x=y;//comment\n"
               "c=d; \n"
               "a=b;</script>\n"
+      "<a href=\"http://www.skippage.com/skipped.html\">Skip</a>\n"
       "</body></html>\n";
     
     make_html_file( "./file1.html", page1 );
@@ -1078,19 +1221,16 @@ void test2 (void)
     make_session_file( "." , "200",
                 "#Session for the combined test\n"
                 "Set-Cookie sessioncookie=sesval;path=/\n"
-                "Custom-Pattern attacked Custom-Replacement repaired\n"
-                "Custom-Pattern line should Custom-Replacement line might Custom-Options none\n"
-                "Custom-Pattern (cu[^ ]+om) (replace) (trick) "
-                     "Custom-Replacement ---$1---$2-$2-$2-$3 "
-                     "Custom-Options none\n"
-                "Custom-Pattern Another "
-                     "Custom-Replacement (One More) "
-                     "Custom-Options i "
-                     "Custom-Content-Type html \n"
-                "Custom-Pattern The "
-                     "Custom-Replacement Should-not-have-been-replaced "
-                     "Custom-Options i "
-                     "Custom-Content-Type text/plain \n"
+                "Custom-Replacement attacked . .* repaired\n"
+                "Custom-Replacement line\\sshould . .* line might\n"
+                "Custom-Replacement (cu\\S+om)\\s(replace)\\s(trick) "
+                     ". .* ---$1---$2-$2-$2-$3 \n"
+                "Custom-Replacement AnoTHer i html (One More)\n"
+                "Custom-Replacement The i text/plain Should-not-have-been-replaced\n"
+                "CookieDomainExpansion .wc2.com foo bar\n"
+                "CookieDomainExpansion .wc3.com foo bar\n"
+                "CookieDomainExpansion .wc4.com - foo bar\n"
+                "SkipLink file1.html skipped.html\n"
                      );
     
     // Simple test with nomove, only invokes cproxy filter
@@ -1226,6 +1366,10 @@ void test2 (void)
         "x<y;</script>") );
     YAZ_CHECK( check_response_content(pack,
         "a=b;</script>") );
+
+    // Check that skiplink prevented the proxifying of the link
+    YAZ_CHECK( check_response_content(pack,
+        "http://www.skippage.com/skipped.html") );
     
     delete(pack);
 
@@ -1248,7 +1392,7 @@ void test2 (void)
         "dummyhost.com/testfile/nosuchfile.html",1);
     pack->router(router).move();
     YAZ_CHECK( check_response_code(pack, 404) );
-    YAZ_CHECK( check_response_content(pack, "<TITLE>YAZ" ) );
+    YAZ_CHECK( check_response_content(pack, "<TITLE>Metaproxy" ) );
     YAZ_CHECK( check_response_content(pack, "Error: 404" ) );
     // At the moment the rewrite blindly replaces everything
     YAZ_CHECK( check_response_content(pack, "http://www.w3.org/TR/html4/strict.dtd" ) );
@@ -1260,14 +1404,13 @@ void test2 (void)
         "dummyhost.com/testfile/nosuchfile.html",1);
     pack->router(router).move();
     YAZ_CHECK( check_response_code(pack, 404) );
-    YAZ_CHECK( check_response_content(pack, "<TITLE>YAZ" ) );
+    YAZ_CHECK( check_response_content(pack, "<TITLE>Metaproxy" ) );
     YAZ_CHECK( check_response_content(pack, "Error: 404" ) );
     delete(pack);
     
     // Post request with a bad url, but with a referer header
     // CPXY-81. Should process it directly, and not redirect.
     // Also, makes an entry in the _cookietrace, which we analyze later
-    // TODO: Not coded that way yet (29-Aug-2013)
     std::string contentparams = url_enc("foo=bar") + "&";
     pack = make_package("http://localhost:9036/"
                "cproxydebug-keepcontent-nomove-dump-verbose/"
@@ -1277,7 +1420,7 @@ void test2 (void)
                 "n1=v1;cproxysession=dummyhost.com/200 ; n2=v2");
     pack->router(router).move();
     YAZ_CHECK( check_response_code(pack, 200) );  // most of all, not a 302
-
+    delete pack;
 
     // Test with custom response headers
     std::string rawpage =
@@ -1301,6 +1444,10 @@ void test2 (void)
            "Expires: 31 Dec 2013 12:34:56; Secure \n"
       "Set-Cookie: n9=v; domain=wrong.com\n"
       "Set-Cookie: e=\n"   // empty value, see CPXY-75
+      "Set-Cookie: w1=v; domain=.wc1.com\n" // wildcard expansion
+      "Set-Cookie: w2=v; domain=.wc2.com\n" 
+      "Set-Cookie: w3=v; domain=.wc3.com\n"
+      "Set-Cookie: w4=v; domain=.wc4.com\n"
         "\n"
       "<html><head>"
       "</head>\n"
@@ -1379,6 +1526,27 @@ void test2 (void)
                 "pAtH=/YYY/ZZZ/200/DummyDOMAIN.com/raWtestFILE; "
                 "dOmAiN=localhost; "
                 "Expires: 31 Dec 2013 12:34:56; Secure " ) );
+    YAZ_CHECK( check_any_response_header(pack,"Set-Cookie",
+                "n9=v; "
+                "domain=localhost; "
+                "path=/YYY/ZZZ/200/wrong.com/rawtestfile/rawfile.html") );
+
+    YAZ_CHECK( check_any_response_header(pack,"Set-Cookie",
+                "w1=v; domain=localhost; path=/YYY/ZZZ/200/www.wc1.com/") );
+    YAZ_CHECK( check_any_response_header(pack,"Set-Cookie",
+                "w1=v; domain=localhost; path=/YYY/ZZZ/200/auth.wc1.com/") );
+    YAZ_CHECK( check_any_response_header(pack,"Set-Cookie",
+                "w2=v; domain=localhost; path=/YYY/ZZZ/200/foo.wc2.com/") );
+    YAZ_CHECK( check_any_response_header(pack,"Set-Cookie",
+                "w2=v; domain=localhost; path=/YYY/ZZZ/200/bar.wc2.com/") );
+    YAZ_CHECK( check_any_response_header(pack,"Set-Cookie",
+                "w3=v; domain=localhost; path=/YYY/ZZZ/200/www.wc3.com/") );
+    YAZ_CHECK( check_any_response_header(pack,"Set-Cookie",
+                "w3=v; domain=localhost; path=/YYY/ZZZ/200/foo.wc3.com/") );
+    YAZ_CHECK( !check_any_response_header(pack,"Set-Cookie",  // cleared by the '-'
+                "w4=v; domain=localhost; path=/YYY/ZZZ/200/www.wc4.com/") );
+    YAZ_CHECK( check_any_response_header(pack,"Set-Cookie",
+                "w4=v; domain=localhost; path=/YYY/ZZZ/200/foo.wc4.com/") );
     
     delete(pack);
 
